import Vue, { PluginObject } from 'vue'
import { WinboxUIComponent, WinboxUIComponentSize, WinboxUIHorizontalAlignment } from './component'

import { WAffix } from './affix'
import { WAvatar } from "./avatar"
import { WAnchor } from './anchor'
import { WBacktop } from './backtop'
import { WBadge } from './badge'
import { WBreadcrumb } from './breadcrumb'
import { WBreadcrumbItem } from './breadcrumb-item'
import { WButton } from './button'
import { WButtonGroup } from './button-group'
import { WCheckbox } from './checkbox'
import { WCheckboxButton } from './checkbox-button'
import { WCheckboxGroup } from './checkbox-group'
import { WCollapse } from './collapse'
import { WCollapseItem } from './collapse-item'
import { WClipboard} from './clipboard'
import { WDatePicker } from './date-picker'
import { WDivider } from './divider'
import { WDialog } from './dialog'
import { WDrawer } from './drawer'
import { WDropdown } from './dropdown'
import { WDropdownItem } from './dropdown-item'
import { WDropdownMenu } from './dropdown-menu'
import { WEmpty } from './empty'
import { WForm } from './form'
import { WFormItem } from './form-item'
import { WIcon } from './icon'
import { WInput } from './input'
import { WInputNumber } from './input-number'
import { WImage} from './image'
import { WMessage } from './message'
import { WMessageBox } from './message-box'
import { WNotification } from './notification'
import { WOption } from './option'
import { WOptionGroup } from './option-group'
import { WPagination } from './pagination'
import { WPopover } from './popover'
import { WPopconfirm } from './popconfirm'
import { WRadio } from './radio'
import { WRadioButton } from './radio-button'
import { WRadioGroup } from './radio-group'
import { WResult } from './result'
import { WSelect } from './select'
import { WScrollbar } from './scrollbar'
import { WTabs } from './tabs'
import { WTabPane } from './tab-pane'
import { WTag } from './tag'
import { WTimePicker } from './time-picker'
import { WTimeSelect } from './time-select'
import { WTooltip } from './tooltip'
import { WTextEllipsis} from './text-ellipsis'
import { WTree, TreeData } from './tree'
import { WTable } from './table'
import { WTableColumn } from './table-column'
import { WSwitch } from './switch'
import { WMenu } from './menu'
import { WMenuItem } from './menu-item'
import { WSubmenu } from './submenu'
import { WMenuItemGroup } from './menu-item-group'
import { WCascader } from './cascader'
import { WCascaderPanel } from './cascader-panel'
import { WUpload } from './upload'
import { WProgress } from './progress'
import { WLink } from './link'
import { WLoading } from './loading'
import { WContainer } from './container'
import { WHeader } from './header'
import { WAside } from './aside'
import { WMain } from './main'
import { WFooter } from './footer'
import { WTimeline } from './timeline'
import { WTimelineItem } from './timeline-item'
import { WAlert } from './alert'
import { WSteps } from './steps'
import { WStep } from './step'
import { WList } from './list'
import { WListItem } from './list-item'
import { WListItemMeta } from './list-item-meta'
import { WInfiniteScroll } from './infinite-scroll'
import { WColorPicker } from './color-picker'
import { WTreeSelect, TreeProps } from './tree-select'
import { WDockingLayout } from './docking-layout'
import { WCard } from './card'
import { WPageHeader } from './page-header'
import { WSlider } from './slider'
import { WRate } from './rate'

import { WTransfer } from './transfer'

export interface InstallationOptions {
  locale: any,
  i18n: any,
  size: string
}

/** The version of winbox-ui */
export const version: string

/**
 * Install all winbox-ui components into Vue.
 * Please do not invoke this method directly.
 * Call `Vue.use(WinboxUI)` to install.
 */
 export function install (vue: typeof Vue, options: InstallationOptions): void

 /** WinboxUI component common definition */
 export type Component = WinboxUIComponent
 
 /** Component size definition for button, input, etc */
 export type ComponentSize = WinboxUIComponentSize
 
 /** Horizontal alignment */
 export type HorizontalAlignment = WinboxUIHorizontalAlignment

 /** Show animation while loading data */
export const Loading: WLoading

/** Used to show feedback after an activity. The difference with Notification is that the latter is often used to show a system level passive notification. */
export const Message: WMessage

/** A set of modal boxes simulating system message box, mainly for message prompt, success tips, error messages and query information */
export const MessageBox: WMessageBox

/** Displays a global notification message at the upper right corner of the page */
export const Notification: WNotification

// TS cannot merge imported class with namespace, so declare subclasses instead

/** Affix Component */
export class Affix extends WAffix {}

/** Avatar Component */
export class Avatar extends WAvatar {}

/** Anchor Component */
export class Anchor extends WAnchor {}

/** Backtop Component */
export class Backtop extends WBacktop {}

/** Bagde Component */
export class Badge extends WBadge {}

/** Breadcrumb Component */
export class Breadcrumb extends WBreadcrumb {}

/** Breadcrumb Item Component */
export class BreadcrumbItem extends WBreadcrumbItem {}

/** Button Component */
export class Button extends WButton {}

/** Button Group Component */
export class ButtonGroup extends WButtonGroup {}

/** Checkbox Component */
export class Checkbox extends WCheckbox {}

/** Checkbox Button Component */
export class CheckboxButton extends WCheckboxButton {}

/** Checkbox Group Component */
export class CheckboxGroup extends WCheckboxGroup {}

/** Collapse Component */
export class Collapse extends WCollapse {}

/** Collapse Item Component */
export class CollapseItem extends WCollapseItem {}

/** Clipboard Item Component */
export class Clipboard extends WClipboard {}

/** Date Picker Component */
export class DatePicker extends WDatePicker {}

/** Dialog Component */
export class Dialog extends WDialog {}

/** Divider Component */
export class Divider extends WDivider {}

/** Drawer Component */
export class Drawer extends WDrawer {}

/** Dropdown Component */
export class Dropdown extends WDropdown {}

/** Dropdown Item Component */
export class DropdownItem extends WDropdownItem {}

/** Dropdown Menu Component */
export class DropdownMenu extends WDropdownMenu {}

/** Empty Component */
export class Empty extends WEmpty {}

/** Form Component */
export class Form extends WForm {}

/** Form Item Component */
export class FormItem extends WFormItem {}

/** Icon Component */
export class Icon extends WIcon {}

/** Input Component */
export class Input extends WInput {}

/** Input Number Component */
export class InputNumber extends WInputNumber {}

/** Image Component */
export class Image extends WImage {}

/** Dropdown Select Option Component */
export class Option extends WOption {}

/** Dropdown Select Option Group Component */
export class OptionGroup extends WOptionGroup {}

/** Pagination Component */
export class Pagination extends WPagination {}

/** Popover Component */
export class Popover extends WPopover {}

/** Popconfirm Component */
export class Popconfirm extends WPopconfirm {}

/** Radio Component */
export class Radio extends WRadio {}

/** Radio Button Component */
export class RadioButton extends WRadioButton {}

/** Radio Group Component */
export class RadioGroup extends WRadioGroup {}

/** Result Component */
export class Result extends WResult {}

/** Dropdown Select Component */
export class Select extends WSelect {}

/** Scrollbar Component */
export class Scrollbar extends WScrollbar {}

/** Tabs Component */
export class Tabs extends WTabs {}

/** Tab Pane Component */
export class TabPane extends WTabPane {}

/** Tag Component */
export class Tag extends WTag {}

/** TimePicker Component */
export class TimePicker extends WTimePicker {}

/** TimeSelect Component */
export class TimeSelect extends WTimeSelect {}

/** Tooltip Component */
export class Tooltip extends WTooltip {}

/** TextEllipsis Component */
export class TextEllipsis extends WTextEllipsis {}

/** Tree Component */
export class Tree<K = any, D = TreeData> extends WTree<K, D> {}

/** Table Component */
export class Table extends WTable {}

/** Table-column Component */
export class TableColumn extends WTableColumn {}

/** Switch Component */
export class Switch extends WSwitch {}

/** Menu Component */
export class Menu extends WMenu {}

/** MenuItem Component */
export class MenuItem extends WMenuItem {}

/** Submenu Component */
export class Submenu extends WSubmenu {}

/** MenuItemGroup Component */
export class MenuItemGroup extends WMenuItemGroup {}

/** Cascader Component */
export class Cascader extends WCascader {}

/** CascaderPanel Component */
export class CascaderPanel extends WCascaderPanel {}

/** Upload Component */
export class Upload extends WUpload {}

/** Progress Component */
export class Progress extends WProgress {}

/** Link Component */
export class Link extends WLink {}

/** Container Component */
export class Container extends WContainer {}

/** Header Component */
export class Header extends WHeader {}

/** Aside Component */
export class Aside extends WAside {}

/** Main Component */
export class Main extends WMain {}

/** Footer Component */
export class Footer extends WFooter {}

/** Timeline Component */
export class Timeline extends WTimeline {}

/** TimelineItem Component */
export class TimelineItem extends WTimelineItem {}

/** Alert Component */
export class Alert extends WAlert {}

/** Steps Component */
export class Steps extends WSteps {}

/** Step Component */
export class Step extends WStep {}

/** List Component */
export class List extends WList {}

/** ListItem Component */
export class ListItem extends WListItem {}

/** List Component */
export class ListItemMeta extends WListItemMeta {}

/** InfiniteScroll Directive */
export const InfiniteScroll: PluginObject<WInfiniteScroll>

/** ColorPicker Component */
export class ColorPicker extends WColorPicker {}

/** TreeSelect Component */
export class TreeSelect<K = any, D = TreeProps> extends WTreeSelect<K, D> {}

/** DockingLayout Component */
export class DockingLayout extends WDockingLayout {}

/** Card Component */
export class Card extends WCard {}

/** PageHeader Component */
export class PageHeader extends WPageHeader {}

/** Slider Component */
export class Slider extends WSlider {}

/** Rate Component */
export class Rate extends WRate {}

/** Transfer Component */
export class Transfer extends WTransfer {}
